<?php
namespace PME\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

class PratoController extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $connection;
	protected $environment;
	protected $pratoService;

    public function __construct($crudService, $parameterBag, $connection, $environment, $pratoService) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->environment = $environment;
        $this->pratoService = $pratoService;
	}
	
	private function callSave($dataSourceName, $rowsToSave){
	    $dataSet = new DataSet($dataSourceName, $rowsToSave);
	    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
	    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
	    $this->crudService->save($dataSourceName, $rowsToSave);
	}
  
	public function saveRecipe(DTO\Request $request, DTO\Response $response) {
	    try {
		    $this->connection->beginTransaction();
	
	        $rows = $request->getRow();
	        $rowsToSave = [];
	        
	        switch($row['TIPORECEITA']){
                case 'S':
                    $this->dataSourceName = 'receserv';
                break;
                case 'L':
                    $this->dataSourceName = 'recefili';
                break;
                case 'P':
                    $this->dataSourceName = 'recepadr';
                break;
            }
	        foreach($rows as $row) {
		    	if (array_key_exists("CDPRODUTO_AUX", $row)) { // atualização
	                if ($row['CDPRODUTO'] <> $row['CDPRODUTO_AUX']) {
	                    $row['__is_new'] = false;
	                    $produtoNovo = $row['CDPRODUTO'];
	                    $row['CDPRODUTO'] = $row['CDPRODUTO_AUX'];
	
	                    $this->delete($request, $response);
	                    if ($response->getError()) {
	                        throw new \Exception($response->getError()->getMessage());
	                    }
	                    $this->pratoService->flush();
	
	                    $row['CDPRODUTO'] = $produtoNovo;
	                    $row['__is_new'] = true;
	                }
	            }
	        
	            $conditions = array(
	                array(
	                    "columnName" => "CDPRATO",
	                    "operator" => "=",
	                    "value" => $row['CDPRATO']
	                ),
	                array(
	                    "columnName" => "CDFILAUXPRAT",
	                    "operator" => "=",
	                    "value" => $row['CDFILAUXPRAT']
	                ),
	                array(
	                    "columnName" => "CDPRODUTO",
	                    "operator" => "=",
	                    "value" => $row['CDPRODUTO']
	                )
	            );
	     
	            if($row['TIPORECEITA'] == 'S' || $row['TIPORECEITA'] == 'L'){
	                if($row['TIPORECEITA'] == 'S'){
	                    array_push($conditions, 
	                        array(
	                            "columnName" => "CDSERVFILI",
	                            "operator" => "=",
	                            "value" => $row['CDSERVFILI']
	                        )
	                    );
	                }
	                array_push($conditions, 
	                    array(
	                        "columnName" => "CDFILIAL",
	                        "operator" => "=",
	                        "value" => $row['CDFILIAL']
	                    )
	                );
	            }
	            
	            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($this->dataSourceName, $conditions);
	            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
	            $receita = $dataSet->getRows();
	            if($receita && $row['__is_new']){
	                $response->addMessage(new DTO\Response\Message('Já Cadastrado!'));
	            }else{
            		array_push($rowsToSave, $row);
	            }
	        }
	        $this->callSave($this->dataSourceName, $rowsToSave);
        	$this->connection->commit();
        	$response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
	}
	
	public function saveDataRecipeStandard(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	   		$rows = $this->crudService->getRowsToSave($request, 'pratpadr');
	        $this->crudService->save('pratpadr', $rows);
	    } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
	}
	
	public function saveDataRecipeLocal(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	    	$rows = $this->crudService->getRowsToSave($request, 'pratfili');
	        $this->crudService->save('pratfili', $rows);
	    } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
	}
	
	public function saveDataRecipeService(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	    	$rows = $this->crudService->getRowsToSave($request, 'pratserv');
	        $this->crudService->save('pratserv', $rows);
	    } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
	}
	
	public function retornaFatorConversaoProduto(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $produto = $row['CDPRODUTO'];
        $sgunidade = $row['SGUNIDADE'];
        $nrorg = $this->environment->getNrOrgTrab();

        $result = $this->pratoService->retornaFatorConversao($produto, $sgunidade, $nrorg);
        $response->addDataset(new DataSet('INFOUNIDADE', $result[0]));
    }
    
     public function retornaFatorConversaoPrato(DTO\Request\Row $request, DTO\Response $response)
     {
        $row = $request->getRow();

        $produtosRec = $row['PRODUTOS'];
        $sgunidadeCompar = $row['SGUNIDADEPRAT'];
        $qtAtiva = $row['QTATIVA'];
        $nrorg = $this->environment->getNrOrgTrab();
        $qtd = 0;

        foreach ($produtosRec as $produto) {
            $valorQtd = $produto[$qtAtiva];
            $sgUnidProd = $produto['SGUNIDADE'] ? $produto['SGUNIDADE'] : $produto['SGUNIDADEPROD'];
            if ($sgUnidProd != $sgunidadeCompar) {
                $fatorConv = $this->pratoService->retornaFatorConversaoPrato(
                    $produto['CDPRODUTO'], $sgUnidProd,
                    $sgunidadeCompar, $nrorg
                );
                if(!empty($fatorConv)) {
                    if ($fatorConv[0]['VRFATOCONVES'] === 0) {
                        $qtd = $qtd + $valorQtd;
                    } else {
                        $invertValor = ($fatorConv[0]['ORDEM'] === 3);
        
                        if ($fatorConv[0]['IDTIPOPE'] === '*') {
                            if ($invertValor === true) {
                                $qtd = $qtd + ($valorQtd / $fatorConv[0]['VRFATOCONVES']);
                            } else {
                                $qtd = $qtd + ($valorQtd * $fatorConv[0]['VRFATOCONVES']);
                            }
                        } else {
                            if ($invertValor === true) {
                                $qtd = $qtd + ($valorQtd * $fatorConv[0]['VRFATOCONVES']);
                            } else {
                                $qtd = $qtd + ($valorQtd / $fatorConv[0]['VRFATOCONVES']);
                            }
                        }
                    }
                }
            } else {
                $qtd = $qtd + $valorQtd;
            }
        }
        $response->addDataset(new DataSet_Source('QTPESSOA', array($qtd)));
    }
    
    public function retornaParametrosAdicionarPrato(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $this->authController->validationDependingGroup($request, $response);
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramgeral', array());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);
    }
}
