<?php

/**
 * 20/10/2015
 * @author Victor Novy
 */

namespace Teknisa\PebLibs\Services;

use \Teknisa\PebLibs\Services\DateUtil;
use \Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Movilote;
use \Geral\Entities\Produto;
use \Estoque\Entities\Posestdia;
use \Estoque\Entities\Itlanctoest;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Prodfili;

class ValidacoesEstoqueService {
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'E' para 'Lançamento de Entrada' */
    const ENTRADA       = 'E';
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'S' para 'Lançamento de Saida'   */
    const SAIDA         = 'S';
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'A' para 'Ajuste de Inventario'
     * e 'Implantacao de Estoque'
     */
    const AJUSTE_IMP    = 'A';
    const TIPO_CUSTO_PADRAO = '03';
    const ESTOQUE_VAZIO = ' ';

    protected $entityManager;
    protected $crudService;
    protected $environment;
    protected $irregularProducts = array();

    private $lanctoestoq;

    public function __construct (EntityManager $entityManager, $crudService, $environment) {
        $this->entityManager = $entityManager;
        $this->crudService = $crudService;
        $this->environment = $environment;
    }

    /**
     * Realiza a validacao da data de lancemento com base nas parametrizacoes feitas pelo cliente.
     * Estas parametrizacoes podem ser encontradas na
     * Tela => 'Parametros de Estoque' => Tabela: 'PARAMGERAL' E 'PARAMESTOQUE'.
     * Essas validacoes servem para controlar as datas que serao efetuados os lancamentos.
     *
     * periodo valido¹ - Este periodo seria a 'data de processamento' da filial menos e/ou mais a quantidade de dias
     * que esta parametrizado
     *
     * @param $filial
     * @param $dtLancamento
     * @param $operacao
     * @param $sistema
     * @param $dtEmissao
     * @throws \Exception
     */
    public function validaDataLancamento($filial, $dtLancamento, $operacao, $sistema, $dtEmissao) {
        try {
            $organizacao = $this->environment->getNrOrgTrab();

            $dadosProcessFilial = $this->retornaDadosProcessFilial($filial, $organizacao);

            $dataProcess = $dadosProcessFilial['DTPROCESSA'];

            $utDtProcessAnt = $dadosProcessFilial['IDLANCDTMEN'];
            $utDtProcessPos = $dadosProcessFilial['IDLANCDTMAI'];
            $utDtProcessAntFilial = $dadosProcessFilial['IDLANCDTMENFIL'];
            $utDtProcessPosFilial = $dadosProcessFilial['IDLANCDTMAIFIL'];

            $qtDiasLancAntEntrada = $dadosProcessFilial['NRDIASENTR'];
            $qtDiasLancAntEntradaFilial = $dadosProcessFilial['NRDIASENTRFIL'];
            $qtDiasLancPostEntrada = $dadosProcessFilial['NRDIASENTRPD'];
            $qtDiasLancPostEntradaFilial = $dadosProcessFilial['NRDIASENTRPDFIL'];

            $qtDiasLancAntSaida = $dadosProcessFilial['NRDIASSAID'];
            $qtDiasLancAntSaidaFilial = $dadosProcessFilial['NRDIASSAIDFIL'];
            $qtDiasLancPostSaida = $dadosProcessFilial['NRDIASSAIDPD'];
            $qtDiasLancPostSaidaFilial = $dadosProcessFilial['NRDIASSAIDPDFIL'];

            $qtDiasLancAntAjuste = $dadosProcessFilial['NRDIASAJUS'];
            $qtDiasLancPostAjuste = $dadosProcessFilial['NRDIASAJUSPD'];
            $qtDiasLancAntAjusteFilial = $dadosProcessFilial['NRDIASAJUSFIL'];
            $qtDiasLancPostAjusteFilial = $dadosProcessFilial['NRDIASAJUSPDFIL'];

            $dtLancamento = DateUtil::getDataDeString($dtLancamento, DateUtil::FORMATO_BRASILEIRO, true);
            $dtEmissao = DateUtil::getDataDeString($dtEmissao, DateUtil::FORMATO_BRASILEIRO, true);
            $dataProcess = DateUtil::getDataDeString(DateUtil::truncateString($dataProcess), DateUtil::FORMATO_BRASILEIRO, true);

            $isDtLancAfterDtProcess = DateUtil::comparaDatas($dtLancamento, $dataProcess, DateUtil::DEPOIS);
            $isEqual = DateUtil::comparaDatas($dtLancamento, $dataProcess, DateUtil::IGUAL);

            if ($utDtProcessAnt === 'N' && $utDtProcessAntFilial === 'N' && !($isDtLancAfterDtProcess || $isEqual))
                throw new \Exception ('Data de lançamento deve ser maior ou igual à data de processamento.');

            if ($utDtProcessPos === 'N' && $utDtProcessPosFilial === 'N' && !(!$isDtLancAfterDtProcess || $isEqual))
                throw new \Exception ('Data de lançamento deve ser menor ou igual à data de processamento.');

            switch ($operacao) {
                case self::ENTRADA :
                    if(!$isDtLancAfterDtProcess) {
                        $this->validaPeriodoLancamento($utDtProcessAntFilial == 'S'? $utDtProcessAntFilial: $utDtProcessAnt, $dtLancamento, $dataProcess, $utDtProcessAntFilial == 'S'? $qtDiasLancAntEntradaFilial : $qtDiasLancAntEntrada );
                    }else {
                        $this->validaPeriodoLancamento($utDtProcessPosFilial == 'S'? $utDtProcessPosFilial: $utDtProcessPos, $dtLancamento, $dataProcess, $utDtProcessPosFilial == 'S'? $qtDiasLancPostEntradaFilial : $qtDiasLancPostEntrada, DateUtil::DEPOIS);
                    }

                    if (DateUtil::comparaDatas($dtEmissao, $dtLancamento, DateUtil::DEPOIS))
                        throw new \Exception('Data de entrada deve ser maior ou igual a data emissão.');

                    break;
                case self::SAIDA :
                    if ($sistema <> 'FAI' || $sistema <> 'FAC' || $sistema <> 'FAZ'){
                        
                        if (!$isDtLancAfterDtProcess) {
                            $this->validaPeriodoLancamento($utDtProcessAntFilial == 'S'? $utDtProcessAntFilial: $utDtProcessAnt, $dtLancamento, $dataProcess, $utDtProcessAntFilial == 'S'? $qtDiasLancAntSaidaFilial : $qtDiasLancAntSaida);
                        } else {
                            $this->validaPeriodoLancamento($utDtProcessPosFilial == 'S'? $utDtProcessPosFilial : $utDtProcessPos, $dtLancamento, $dataProcess, $utDtProcessPosFilial == 'S'? $qtDiasLancPostSaidaFilial : $qtDiasLancPostSaida, DateUtil::DEPOIS);
                        }
                    }
                    break;
                case self::AJUSTE_IMP :
                    
                    if (!$isDtLancAfterDtProcess) {
                        $this->validaPeriodoLancamento($utDtProcessAntFilial == 'S'? $utDtProcessAntFilial : $utDtProcessAnt, $dtLancamento, $dataProcess, $utDtProcessAntFilial == 'S'? $qtDiasLancAntAjusteFilial : $qtDiasLancAntAjuste);
                    } else {
                        $this->validaPeriodoLancamento($utDtProcessPosFilial == 'S'? $utDtProcessPosFilial : $utDtProcessPos, $dtLancamento, $dataProcess, $utDtProcessPosFilial == 'S'? $qtDiasLancPostAjusteFilial : $qtDiasLancPostAjuste, DateUtil::DEPOIS);
                    }
                    break;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Validação do tipo 'Lancamento Entrada'
     * Verifica se a data do lancamento que esta sendo realizado esta dentro do perido valido¹ e se 'data de emissão'
     * não é anterior a 'data do lancamento'.
     *
     * @param $dtLancamento
     * @param $dataProcess
     * @param $utDtProcessAnt
     * @param $qtDiasLancAntEntrada
     * @param $utDtProcessPos
     * @param $qtDiasLancPostEntrada
     * @param $utDtProcessAntFilial
     * @param $qtDiasLancAntEntradaFilial
     * @param $utDtProcessPosFilial
     * @param $qtDiasLancPostEntradaFilial
     * @param $dtEmissao
     * @param $isDtLancAfterDtProcess
     * @param $isEqual
     * @throws \Exception
     */
    private function validaDataLancamentoEntrada(
        $dtLancamento, $dataProcess, $utDtProcessAnt, $qtDiasLancAntEntrada, $utDtProcessPos,
        $qtDiasLancPostEntrada, $utDtProcessAntFilial, $qtDiasLancAntEntradaFilial,
        $utDtProcessPosFilial, $qtDiasLancPostEntradaFilial, $dtEmissao,
        $isDtLancAfterDtProcess, $isEqual
    ) {
        if(!$isDtLancAfterDtProcess) {
            $this->validaPeriodoLancamento($utDtProcessAnt, $dtLancamento, $dataProcess, $qtDiasLancAntEntrada);
            $this->validaPeriodoLancamento($utDtProcessAntFilial, $dtLancamento, $dataProcess, $qtDiasLancAntEntradaFilial);
        } else {
            $this->validaPeriodoLancamento($utDtProcessPos, $dtLancamento, $dataProcess, $qtDiasLancPostEntrada, DateUtil::DEPOIS);
            $this->validaPeriodoLancamento($utDtProcessPosFilial, $dtLancamento, $dataProcess, $qtDiasLancPostEntradaFilial, DateUtil::DEPOIS);
        }

        if (DateUtil::comparaDatas($dtEmissao, $dtLancamento, DateUtil::DEPOIS))
            throw new \Exception('Data de entrada deve ser maior ou igual a data emissão.');
    }

    /**
     * Validação do tipo 'Lancamento Saida'
     * Verifica se a data do lancamento que esta sendo realizado esta dentro do perido valido¹.
     *
     * @param $dtLancamento
     * @param $dataProcess
     * @param $utDtProcessAnt
     * @param $qtDiasLancAntSaida
     * @param $utDtProcessPos
     * @param $qtDiasLancPostSaida
     * @param $utDtProcessAntFilial
     * @param $qtDiasLancAntSaidaFilial
     * @param $utDtProcessPosFilial
     * @param $qtDiasLancPostSaidaFilial
     * @param $isDtLancAfterDtProcess
     * @param $isEqual
     * @throws \Exception
     */
    private function validaDataLancamentoSaida(
        $dtLancamento, $dataProcess, $utDtProcessAnt, $qtDiasLancAntSaida, $utDtProcessPos,
        $qtDiasLancPostSaida, $utDtProcessAntFilial, $qtDiasLancAntSaidaFilial,
        $utDtProcessPosFilial, $qtDiasLancPostSaidaFilial, $isDtLancAfterDtProcess, $isEqual
    ){
        if (!$isDtLancAfterDtProcess) {
            $this->validaPeriodoLancamento($utDtProcessAnt, $dtLancamento, $dataProcess, $qtDiasLancAntSaida);
            $this->validaPeriodoLancamento($utDtProcessAntFilial, $dtLancamento, $dataProcess, $qtDiasLancAntSaidaFilial);
        } else {
            $this->validaPeriodoLancamento($utDtProcessPos, $dtLancamento, $dataProcess, $qtDiasLancPostSaida, DateUtil::DEPOIS);
            $this->validaPeriodoLancamento($utDtProcessPosFilial, $dtLancamento, $dataProcess, $qtDiasLancPostSaidaFilial, DateUtil::DEPOIS);
        }
    }

    /**
     * Validação do tipo 'Lancamento Saida'
     * Verifica se a data do lancamento que esta sendo realizado esta dentro do perido valido¹.
     *
     * @param $dtLancamento
     * @param $dataProcess
     * @param $utDtProcessAnt
     * @param $qtDiasLancAntAjuste
     * @param $utDtProcessPos
     * @param $qtDiasLancPostAjuste
     * @param $utDtProcessAntFilial
     * @param $qtDiasLancAntAjusteFilial
     * @param $utDtProcessPosFilial
     * @param $qtDiasLancPostAjusteFilial
     * @param $isDtLancAfterDtProcess
     * @param $isEqual
     * @throws \Exception
     */
    private function validaDataLancamentoAjuste(
        $dtLancamento, $dataProcess, $utDtProcessAnt, $qtDiasLancAntAjuste, $utDtProcessPos,
        $qtDiasLancPostAjuste, $utDtProcessAntFilial, $qtDiasLancAntAjusteFilial,
        $utDtProcessPosFilial, $qtDiasLancPostAjusteFilial, $isDtLancAfterDtProcess, $isEqual
    ){
        if (!$isDtLancAfterDtProcess) {
            $this->validaPeriodoLancamento($utDtProcessAnt, $dtLancamento, $dataProcess, $qtDiasLancAntAjuste);
            $this->validaPeriodoLancamento($utDtProcessAntFilial, $dtLancamento, $dataProcess, $qtDiasLancAntAjusteFilial);
        } else {
            $this->validaPeriodoLancamento($utDtProcessPos, $dtLancamento, $dataProcess, $qtDiasLancPostAjuste, DateUtil::DEPOIS);
            $this->validaPeriodoLancamento($utDtProcessPosFilial, $dtLancamento, $dataProcess, $qtDiasLancPostAjusteFilial, DateUtil::DEPOIS);
        }
    }

    /**
     * Verifica se a data do lancamento que esta sendo realizado esta dentro do perido valido¹.
     *
     * @param $utDtProcessAntPos
     * @param \DateTime $dtLancamento
     * @param \DateTime $dataProcess
     * @param $quantDias
     * @param int $tipoComparacao
     * @throws \Exception
     */
    private function validaPeriodoLancamento(
        $utDtProcessAntPos, \DateTime $dtLancamento, \DateTime $dataProcess,
        $quantDias, $tipoComparacao = DateUtil::ANTES
    ){
        
        if ($tipoComparacao === DateUtil::ANTES)
            $dataProcess = DateUtil::subtraiIntervalo($dataProcess, $quantDias);
        else
            $dataProcess = DateUtil::adicionaIntervalo($dataProcess, $quantDias);
        
        if ($utDtProcessAntPos === 'S' && DateUtil::comparaDatas($dtLancamento, $dataProcess, $tipoComparacao))
            throw new \Exception ('Data de lançamento fora do período permitido.');
    }

    /* ******************** Atualizador de Estoque ******************** */

    /**
     * Função que realiza a execução da atualização de
     * estoque. Possui como comportamento padrão a
     * atualização de todos os itens de um lançamento de
     * uma unidade, com verificação de irregularidades e
     * abortamento das operações no caso de invalidez.
     *
     * Permite porém que a verificação de irregularidades
     * não seja efetuada ou que as operações não sejam
     * abortadas após a verificação através da passagem
     * dos parâmetros "verifyIrregularities" e
     * "abortAtIrregularities". Note que o último não
     * pode ser verdadeiro se o primeiro for falso.
     *
     * Permite também que sejam selecionados um ou mais
     * itens para a execução da atualização de estoque
     * através da passagem de seus códigos na variável
     * "seqItemCodes". Ela é nula no comportamento
     * padrão, o que garante que todos os itens do
     * lançamento sejam atualizados.
     *
     * @throws \Exception
     *
     * @param  string     $cdfilial
     * @param  string     $nrlancestq
     * @param  boolean    $verifyIrregularities
     * @param  boolean    $abortAtIrregularities
     * @param  array      $seqItemCodes
     *
     * @return array
     */
    public function atualizaEstoque(
        $cdfilial, $nrlancestq, $consolidate = false, $verifyIrregularities = true, $abortAtIrregularities = true, $seqItemCodes = null
    ) {
        try {
            $connection = $this->entityManager->getConnection();

            #starts main, outer transaction
            $connection->beginTransaction();

            try {
                #starts first inner transaction
                $connection->beginTransaction();

                $nrorg = $this->environment->getNrOrgTrab();

                $lanctoestoq = $this->buscaLanctoestoq($cdfilial, $nrlancestq, $nrorg);

                #marks all items as appurtenant to stock update
                $this->atualizaTodosItensLancamentoEstoque($cdfilial, $lanctoestoq->getNrlancestq(), $nrorg, $seqItemCodes, $consolidate);

                $dadosProcess = $this->retornaDadosProcessFilial($cdfilial, $nrorg);

                #separate dates
                $dtprocessa = $dadosProcess['DTPROCESSA'];
                $dtlancestq = $lanctoestoq->getDtlancestq();

                #process dates
                $dtprocessa = $this->getDateObject($dtprocessa);
                $dtlancestq = $this->getDateObject($dtlancestq);

                #order dates
                $date = $this->getOrderedDates($dtprocessa, $dtlancestq);

                #updates stock
                $this->executaProcedureAtualizaEstoque($lanctoestoq->getCdfilial(), $lanctoestoq->getCdfilimovi(), $nrlancestq, $date["INITIAL"], $date["FINAL"]);
                
                #concludes first inner transaction
                $connection->commit();
            } catch (\Exception $e) {
                #rollbacks operations in this transaction
                $connection->rollback();

                #throws error to cancel outer transaction
                throw $e;
            }

            #Please note that two separate transactions are
            #required for the correct operation of the function
            #This second step, where the procedure is executed
            #again, only works if the first one has already
            #been commited.

            try {
                #starts second inner transaction
                $connection->beginTransaction(); 

                #checks flags
                if($verifyIrregularities){
                    if($abortAtIrregularities)
                        #verifies irregularities and throws exception if any is found
                        $this->verificaIrregularidades($lanctoestoq, $seqItemCodes);
                    else {
                        #verifies irregularities and updates items for them to be ignored if any is found
                        $this->atualizaItensIrregulares($lanctoestoq, $seqItemCodes);

                        #updates stock with new parametrizations
                        $this->executaProcedureAtualizaEstoque($lanctoestoq->getCdfilial(), $lanctoestoq->getCdfilimovi(), $nrlancestq, $date["INITIAL"], $date["FINAL"]);
                    }
                }

                #concludes second inner transaction
                $connection->commit();
            } catch (\Exception $e) {
                #rollbacks operations in this transaction
                $connection->rollback();
                #throws error to cancel outer transaction
                throw $e;
            }

            $irregularProductCodes = array();
            $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);

            #separates irregular product codes
            foreach ($irregularProducts as $product) {
                $irregularProductCodes[] = $product->getCdproduto();
            }

            #concludes outer transaction and all operations
            $connection->commit();
        } catch (\Exception $e) {
            #rollbacks every operation
            $connection->rollback();
            #throws exception for controller to catch
            throw $e;
        }

        #returns any existent irregular product codes
        return $irregularProductCodes;
    }

    public function getIrregularProducts() {
        return $this->irregularProducts;
    }

    public function retornaProdutosIrregulares(Lanctoestoq $lanctoestoq, $seqItens = null) {
        $nrmovilote = $lanctoestoq->getCdfilial().$lanctoestoq->getNrlancestq();

        $produtosIrregulares = Movilote::findByNrmoviloteAndNrorg(
            $nrmovilote, $lanctoestoq->getNrorg()
        );
        
        if(!is_null($seqItens)) {
            $itlanctoest = $this->buscaItlanctoest(
                $lanctoestoq->getCdfilial(), $lanctoestoq->getNrlancestq(),
                $lanctoestoq->getNrorg()
            );

            $seqProds = array();
            foreach($itlanctoest as $item)
                if(in_array($item->getNrsequitem(), $seqItens))
                    $seqProds[] = $item->getCdproduto();

            $produtosIrregularesInSeqItens = array();

            foreach ($produtosIrregulares as $produto)
                if(in_array($produto->getCdproduto(), $seqProds))
                    $produtosIrregularesInSeqItens[] = $produto;

            return $produtosIrregularesInSeqItens;
        }

        /** @var \Estoque\Entities\Movilote $produtosIrregulares */
        return $produtosIrregulares;
    }

    /**
     * Procedimento que verifica se o estoque apresenta
     * alguma irregularidade, lançando uma exceção caso
     * alguma seja encontrada.
     *
     * @throws \Exception
     *
     * @param  Lanctoestoq $lanctoestoq
     * @param  array       $seqItens
     */
    public function verificaIrregularidades(Lanctoestoq $lanctoestoq, $seqItemCodes = null) {
        $this->irregularProducts = array();
        #checks for irregularities
        $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);

        #TODO Verify a better message
        if (!empty($irregularProducts)) {
            $this->irregularProducts = $irregularProducts;
            throw new \Exception ('A confirmação da operação faria com que a posição de estoque de alguns itens se tornasse irregular.');
            
        }
    }

    /**
     * Procedimento que atualiza os itens irregulares de
     * um lançamento de estoque, retirando-os do conjunto
     * de itens pertencentes à evolução de estoque.
     *
     * @param  Lanctoestoq $lanctoestoq
     * @param  array       $seqItens
     */
    public function atualizaItensIrregulares(Lanctoestoq $lanctoestoq, $seqItemCodes = null) {
        #checks for irregularities
        $irregularProducts = $this->retornaProdutosIrregulares($lanctoestoq, $seqItemCodes);

        #if there's any
        if(!empty($irregularProducts)) {
            #separates irregular product codes
            $productCodes = array();
            foreach($irregularProducts as $product)
                $productCodes[] = $product->getCdproduto();

            #gets possibly irregular items
            $itlanctoest = $this->buscaItlanctoest(
                $lanctoestoq->getCdfilial(), $lanctoestoq->getNrlancestq(),
                $lanctoestoq->getNrorg(), 'S'
            );

            if(!is_null($itlanctoest)) {
                $seqItemCodes = array();
                foreach($itlanctoest as $item)
                    #checks every item, searching for irregular products
                    if(in_array($item->getCdproduto(), $productCodes))
                        #separes item codes that are related to irregular products
                        $seqItemCodes[] = $item->getNrsequitem();

                #updates irregular items
                $this->atualizaItensLancamentoEstoque($itlanctoest, 'N', $seqItemCodes);
            }
        }
    }

    /**
     * Procedimento que atualiza os itens de um
     * lançamento para a evolução/atualização de estoque.
     *
     * Permite a passagem de um parâmetro que define
     * quais itens dentre os recebidos podem/devem ser
     * atualizados.
     *
     * @param array  $itlanctoest
     * @param string $idevolestoq
     * @param array  $seqItemCodes
     */
    private function atualizaItensLancamentoEstoque($itlanctoest, $idevolestoq = 'S', $seqItemCodes = null, $consolidate = false) {
        $itlanctoest = $this->consolidadeToStockProducts($itlanctoest, false);
        
        //Após primeira consolidação de todos produtos de compra em um só, consolida em produto de estoque caso escolhido pelo usuario em tela.
        
        if($consolidate)
            $itlanctoest = $this->consolidadeToStockProducts($itlanctoest, true);
            
        #iterates over every received item
        foreach ($itlanctoest as $item) {
            #checks whether item should be updated
            if(is_null($seqItemCodes) || in_array($item->getNrsequitem(), $seqItemCodes)) {
                #updates item and persists information
                $item->setIdevolestoq($idevolestoq);
                $this->entityManager->persist($item);
                $this->entityManager->flush($item);
                if($idevolestoq == 'S')
                    $this->atualizaProdlcestoq($item->getCdfilimovi(), $item->getCdalmoxarife(), $item->getCdlocalestoq(), $item->getCdproduto());
            }
            if(is_null($item->getIdtipomovi())){
                $item->setIdtipomovi('1');
                $this->entityManager->persist($item);
                $this->entityManager->flush($item);
            }
        }
    }

    /**
     * Procedimento que atualiza todos os itens de um
     * lançamento de estoque independentemente de sua
     * regularidade para que participem na evolução de
     * estoque, respeitando os códigos definidos no
     * parâmetros $seqItemCodes. Se este for omitido,
     * todos os itens serão atualizados sem discriminação
     *
     * @param  string $cdfilial
     * @param  string $nrlancestq
     * @param  string $nrorg
     * @param  array  $seqItemCodes
     */
    private function atualizaTodosItensLancamentoEstoque($cdfilial, $nrlancestq, $nrorg, $seqItemCodes = null, $consolidate = false) {
        //$organizacao = $this->environment->getNrOrgTrab();
        #gets all items from the received stock launch
        $itlanctoest = $this->buscaItlanctoest($cdfilial, $nrlancestq, $nrorg, 'N');

		//remove produtos que nao controlam estoque do array para que nao sejam atualizados
        /*for($i = 0 ; $i< count($itlanctoest) ; $i ++){
            $prodfili = Prodfili::findOneByCdfilialAndCdoridutoAndNrorg($cdfilial, $itlanctoest[$i]->getCdprodmovi(), $organizacao);
            if($prodfili->getIdcntrestoq() == 'N')
                array_slice($itlanctoest, $i, 1);
        }*/ //este controle de produto com controle de estoque está sendo feito via procedure atualmente. Caso necessário descomentar
        
        #updates every item
        if(!is_null($itlanctoest))
            $this->atualizaItensLancamentoEstoque($itlanctoest, 'S', $seqItemCodes, $consolidate);
    }

    /**
     * Procedimento que executa a procedure de
     * atualização de estoque.
     *
     * @param  string $cdfilial
     * @param  string $nrlancto
     * @param  date   $dtinicial
     * @param  date   $dtfinal
     */
    public function executaProcedureAtualizaEstoque($cdfilial, $cdfilimovi, $nrlancto, $dtinicial, $dtfinal) {
        #creates procedure
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_ESTOQUE');
        
        #sets all parameters
        $procedure->addParam(new Param('P_FILIMOVI', Param::PARAM_INPUT, $cdfilimovi, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILILANCTO', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTO', Param::PARAM_INPUT, $nrlancto, Param::PARAM_TYPE_STR));

        #execute procedure
        $procedure->execute();
    }

    /**
     * ZERAR ESTOQUE
     */
    public function zeraInventario($filial,$lancamento,$idOrdemClc, $consolidate = false)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $filialLogada = $this->environment->getCdFilial();
            $cdoperador = $this->environment->getCdOperador();

            $organizacao = $this->environment->getNrOrgTrab();
            $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($filial, $lancamento, $organizacao);

            if (!is_object($lanctoestoq))
                throw new \Exception ('Número do lançamento não encontrado.');

            $dtLancamento = $lanctoestoq->getDtlancestq()->format('d/m/Y');
            $this->executaProcedureAjuteZeraInve(
                $lanctoestoq->getCdfilimovi(),$dtLancamento,$lancamento,$filialLogada,$cdoperador,$lanctoestoq->getIdtplancto(),$idOrdemClc,$lanctoestoq->getCdalmoxarife(),$lanctoestoq->getCdlocalestoq()
            );
            $this->atualizaEstoque($lanctoestoq->getCdfilial(), $lancamento, $consolidate);

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function executaProcedureAjuteZeraInve(
        $filial,$dataLancto, $lancamento,$filialLogada,$operadorLogado,$idtipomovi,$idordemclc,$cdalmoxarife,$cdlocalestoq
    ) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'AJUSTE_ZERA_INVE');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT,   $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTIMPLANT', Param::PARAM_INPUT,  $dataLancto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILILANCTO', Param::PARAM_INPUT, $filialLogada, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $lancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operadorLogado, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTIPOMOVI', Param::PARAM_INPUT, $idtipomovi, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORDEMCLC', Param::PARAM_INPUT, $idordemclc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $cdalmoxarife, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $cdlocalestoq, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

    /* ******************** Consulta o Custo e a Quantidade do Produto ******************** */

    public function consultaCustoQuantidadeProduto(
        $filial, $produto, $dtLancamento, $tipoCusto = self::TIPO_CUSTO_PADRAO,
        $almoxarifado = self::ESTOQUE_VAZIO, $localEstoque = self::ESTOQUE_VAZIO,
        $lote = self::ESTOQUE_VAZIO, $subLote = self::ESTOQUE_VAZIO
    ){

        $custoQuantidade = $this->consultaCusto(
            $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
        );
 
        if (!$custoQuantidade)
            $custoQuantidade = array('QTESTOQDIA' => '0', 'VRCUSTOPROD' => '0');

        return $custoQuantidade;
    }

    /* ******************** Funções gerais ******************** */

    public function retornaDadosProcessFilial($filial, $organizacao) {
        $conditions[] = array(
            'columnName' => 'CDFILIAL',
            'operator' => '=',
            'value' => $filial,
        );
        $conditions[] = array(
            'columnName' => 'NRORG',
            'operator' => '=',
            'value' => $organizacao,
        );

        $dataSet = $this->crudService->getFilteredDataSet('paramestoque_processamento', $conditions);
        $rows = $dataSet->getRows();

        if (!count($rows))
            throw new \Exception('Parametrização geral não encontrada.');

        return $rows[0];
    }

    public function setLanctoestoq(Lanctoestoq $lanctoestoq) {
        $this->lanctoestoq = $lanctoestoq;
    }

    public function getLanctoestoq() {
        return $this->lanctoestoq;
    }

    /**
     * Metodo responsável por varificar se existe a entidade "Lanctoestoq" nessa classe,
     * se não existir, realiza a busca da mesma no banco de dados.
     * Este metodo deve ser utilizado quando você já efetuou a busca em outra classe e não necessita
     * de busca-la novamente no "atualizador de estoque".
     *
     * @param $filial
     * @param $lancamento
     * @param $organizacao
     *
     * @return Lanctoestoq
     */
    public function buscaLanctoestoq($filial, $lancamento, $organizacao) {
        $lanctoestoq = $this->getLanctoestoq();

        if (is_object($lanctoestoq)) {
            return $lanctoestoq;
        }
        $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($filial, $lancamento, $organizacao);
        
        if (!is_object($lanctoestoq))
            throw new \Exception ('Número do lançamento não encontrado.');

        $this->setLanctoestoq($lanctoestoq);
        return $lanctoestoq;
    }

    /**
     * Função que retorna os itens de um lançamento de
     * estoque de acordo com os parâmetros recebidos.
     *
     * Permite que sejam trazidos apenas itens que estão
     * ou não marcados para evolução do estoque através
     * da passagem do parâmetro $idevolestoq. Se este for
     * omitido, todos os itens serão buscados.
     *
     * @param  string $cdfilial
     * @param  string $nrlancestq
     * @param  string $nrorg
     * @param  string $idevolestoq
     *
     * @return Itlanctoest
     */
    private function buscaItlanctoest($cdfilial, $nrlancestq, $nrorg, $idevolestoq = null) {
        #checks parameter existence
        if(is_null($idevolestoq))
            #searches for every existent item
            $itlanctoest = Itlanctoest::findBy(array("cdfilial"  => $cdfilial, 
                                                    "nrlancestq" => $nrlancestq, 
                                                    "nrorg"      => $nrorg
                ), array("cdprpduto" => "asc")
            );
        else
            #searches for certain items
            $itlanctoest = Itlanctoest::findBy(array("cdfilial" => $cdfilial, 
                                                     "nrlancestq" => $nrlancestq,
                                                     "idevolestoq" => $idevolestoq,
                                                     "nrorg"       => $nrorg
                ), array("cdproduto" => "asc")
            );

        # @var \Estoque\Entities\Itlanctoest $itlanctoest
        return $itlanctoest;
    }

    /**
     * Função que retorna um objeto Date à partir de uma
     * string pré-formatada.
     *
     * @param  string   $dateString
     *
     * @return DateTime
     */
    public function getDateObject($dateString) {
        #checks whether date variable is already an object
        if(!is_object($dateString)) {
            #evaluates appropriate date format
            $dateFormat = (strlen($dateString) > 10) ? DateUtil::FORMATO_BRASILEIRO_DATAHORA : DateUtil::FORMATO_BRASILEIRO;

            #creates object
            $dateObject = DateUtil::getDataDeString($dateString, $dateFormat, true);
        } else
            $dateObject = $dateString;

        return $dateObject;
    }

    /**
     * Função que recebe duas datas e retorna um array
     * indicando a data anterior ("INITIAL") e a
     * posterior ("FINAL").
     *
     * @param  DateTime $date
     * @param  DateTime $otherDate
     *
     * @return array
     */
    public function getOrderedDates($date, $otherDate) {
        #compares and separates dates in prior and subsequent
        if($date < $otherDate) {
            $firstDate = $date->format('d/m/Y');
            $lastDate = $otherDate->format('d/m/Y');
        } else {
            $firstDate = $otherDate->format('d/m/Y');
            $lastDate = $date->format('d/m/Y');
        }

        return array("INITIAL" => $firstDate, "FINAL" => $lastDate);
    }
    
    public function atualizaProdlcestoq($cdfilial, $cdalmoxarife, $cdlocalestoq, $cdproduto){
        $sql = "INSERT INTO PRODLCESTOQ
                    (CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO)
                    (SELECT :P_CDFILIAL, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_CDPRODUTO
                     FROM PRODFILI
                     WHERE CDFILIAL  = :P_CDFILIAL  AND
                           CDPRODUTO = :P_CDPRODUTO AND
                           NVL(IDCNTRESTOQ, 'S') <> 'N'
                     MINUS
                     SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO
                     FROM PRODLCESTOQ
                     WHERE CDFILIAL     = :P_CDFILIAL     AND
                           CDALMOXARIFE = :P_CDALMOXARIFE AND
                           CDLOCALESTOQ = :P_CDLOCALESTOQ AND
                           CDPRODUTO    = :P_CDPRODUTO)";
               
        $params = array("P_CDFILIAL" => $cdfilial,
                        "P_CDALMOXARIFE" => $cdalmoxarife,
                        "P_CDLOCALESTOQ" => $cdlocalestoq,
                        "P_CDPRODUTO"  => $cdproduto);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);     
    }
    
    public function deleteItlanctoest($cdfilial, $cdproduto, $nrlancestq){
        $sql = "DELETE ITLANCTOEST
                WHERE CDFILIAL   = :P_CDFILIAL
                AND   CDPRODUTO  = :P_CDPRODUTO
                AND   NRLANCESTQ = :P_NRLANCESTQ";
               
        $params = array("P_CDFILIAL" => $cdfilial,
                        "P_CDPRODUTO" => $cdproduto,
                        "P_NRLANCESTQ" => $nrlancestq);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);     
    }
    
    public function consultaCusto(
        $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
    )
    {
        $sql = <<<SQL
        SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, :P_INTPCUSTO, :P_DTLANC, SYSDATE,
               D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
               (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA
          FROM PARAMFILIAL PF, PRODUTO P, POSESTDIA D, LOTE L
         WHERE P.CDPRODUTO = :P_CDPRODUTO
           AND D.CDFILIAL = :P_CDFILIAL
           AND D.CDALMOXARIFE = :P_CDALMOXARIFE
           AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
           AND D.NRLOTEESTQ = :P_NRLOTEESTQ
           AND D.NRSUBLOTE = :P_NRSUBLOTE
           AND D.CDPRODUTO = P.CDPRODESTO
           AND (
                (D.QTESTOQDIA > 0) 
                OR 
                (PF.IDPERMESTNEG = 'S')
            )
           AND PF.CDFILIAL = L.CDFILIAL
           AND D.CDFILIAL = L.CDFILIAL
           AND D.NRLOTEESTQ = L.NRLOTEESTQ
           AND D.NRSUBLOTE = L.NRSUBLOTE
           AND D.CDPRODUTO = L.CDPRODUTO
           AND D.DTPOSIESTQ = (SELECT MAX(B.DTPOSIESTQ)
                                 FROM POSESTDIA B
                                WHERE B.CDFILIAL     = D.CDFILIAL
                                  AND B.CDPRODUTO    = D.CDPRODUTO
                                  AND B.CDALMOXARIFE = D.CDALMOXARIFE
                                  AND B.CDLOCALESTOQ = D.CDLOCALESTOQ
                                  AND B.NRLOTEESTQ   = D.NRLOTEESTQ
                                  AND B.NRSUBLOTE    = D.NRSUBLOTE
                                  AND B.DTPOSIESTQ   <= TO_DATE(:P_DTLANC) + 1)
SQL;

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_INTPCUSTO" => $tipoCusto,
            "P_CDALMOXARIFE" => $almoxarifado,
            "P_CDLOCALESTOQ" => $localEstoque,
            "P_CDPRODUTO" => $produto,
            "P_NRLOTEESTQ" => $lote,
            "P_NRSUBLOTE" => $subLote,
            "P_DTLANC" => $dtLancamento
        );

        try{
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consolidadeToStockProducts($products, $consolidate = false){
        if($consolidate){ 
            //consolidate all procuts into stock products.
            $stockProductsArray = array();
            foreach($products as $keyA => &$productA){
                if($productA->getCdprodmovi() != $productA->getCdproduto()){
                    $this->entityManager->beginTransaction();
                    $newProdEsto = new Itlanctoest();
                    $newProdEsto->setCdfilial($productA->getCdfilial());
                    $newProdEsto->setNrlancestq($productA->getNrlancestq()); 
                    $newProdEsto->setNrsequitem(($this->getMaxNrSeqItemAjuste($productA->getCdfilial(), $productA->getNrlancestq())));
                    $newProdEsto->setIdtipomovi($productA->getIdtipomovi());
                    $newProdEsto->setCdproduto($productA->getCdprodmovi());
                    $newProdEsto->setQttotlancto(0);
                    $newProdEsto->setVrunilancto(0);
                    $newProdEsto->setVrtotlancto(0);
                    $newProdEsto->setDtlancmovi($productA->getDtlancmovi());
                    $newProdEsto->setIdordemclc($productA->getIdordemclc());
                    $newProdEsto->setCdprodmovi($productA->getCdprodmovi());
                    $newProdEsto->setQtlanctoest(0);
                    $newProdEsto->setVrlanctobrut(0);
                    $newProdEsto->setVrlanctoest(0);
                    $newProdEsto->setCdfilimovi($productA->getCdfilimovi());
                    $newProdEsto->setCdalmoxarife($productA->getCdalmoxarife());
                    $newProdEsto->setCdlocalestoq($productA->getCdlocalestoq());
                    $newProdEsto->setNrloteestq($productA->getNrloteestq());
                    $newProdEsto->setNrsublote($productA->getNrsublote());
                    $newProdEsto->setNrorg($productA->getNrorg());
                    $this->entityManager->persist($newProdEsto);
                    $this->entityManager->flush($newProdEsto);
                    $this->entityManager->commit();
                    array_push($stockProductsArray, $newProdEsto);
                }
            }
            
            foreach($stockProductsArray as $stockProd){
                array_unshift($products, $stockProd);
            }
        }
        
        $indexToUnset = array();
        
        foreach($products as $keyA => &$productA){
            foreach($products as $keyB => &$productB){
                if( ($productB->getNrsequitem() != $productA->getNrsequitem()) && 
                    ($productB->getCdprodmovi() == $productA->getCdprodmovi()) &&
                    ($productB->getNrloteestq() == $productA->getNrloteestq()) &&
                    ($productB->getCdalmoxarife() == $productA->getCdalmoxarife()) &&
                    ($productB->getCdlocalestoq() == $productA->getCdlocalestoq()) &&
                    ($productB->getNrsublote() == $productA->getNrsublote()) &&
                    ($keyB > $keyA)
                )
                    {
                        $this->entityManager->beginTransaction();
                        $prodA = Produto::findOneByCdproduto($productA->getCdproduto());
                        $prodB = Produto::findOneByCdproduto($productB->getCdproduto());

                        $productA->setQtlanctoest($productA->getQtlanctoest() + ($productB->getQttotlancto() * $prodB->getVrfatoconv()));
                        $productA->setQttotlancto($productA->getQttotlancto() + (($productB->getQttotlancto() * $prodB->getVrfatoconv()) / ($prodA->getVrfatoconv() == 0? 1 : $prodA->getVrfatoconv())));
                        
                        $this->entityManager->persist($productB);
                        $this->entityManager->flush($productB);
                        $productB->delete();

                        $this->entityManager->commit();
                         
                        array_push($indexToUnset, $keyB);
                    }
                        
            }

        }
        
        foreach($indexToUnset as $key){
            unset($products[$key]);
        }
        
        return $products;
    }
    
    public function getMaxNrSeqItemAjuste($cdfilial, $nrlancestq) {
        $parameters = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCESTQ" => $nrlancestq
        );
        $sql = "SELECT MAX(NRSEQUITEM) + 1 AS NRSEQUITEM
				FROM ITLANCTOEST
				WHERE NRLANCESTQ = :P_NRLANCESTQ
				AND CDFILIAL     = :P_CDFILIAL";
				
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($parameters);
        $maxNrSeqItem = $std->fetchAll();
        
        
        $nrsequitem = ($maxNrSeqItem[0]["NRSEQUITEM"] != null) ? $maxNrSeqItem[0]["NRSEQUITEM"] : 1;
        $nrsequitem = str_pad($nrsequitem, 4, "0", STR_PAD_LEFT);
        
        return $nrsequitem;
    }
    
    public function array_some(array $array, callable $fn) {
        foreach ($array as $value) {
            if($fn($value)) {
                return true;
            }
        }
        return false;
    }
}