import {
  Modal, ModalService, Button,
  moment, Grid, Select,
  IButtonEvent, Date as ZdDate, Radio,
  Form,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00602ImportarCardDia {
    private modalImportCard!: Modal;
    
    private modalImportacao!: Modal;
    
    private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
    
    public utilities = Loader.getInstance('Utilities');
    
    public colsGridPara = 12;
    
    public showGridPrato = false;
    
    public showArredondar = false;
    
    public NRCARDPMEORIGEM = '';
    
    private datasourceDePara: IDictionary[] = [];
    
    // Funçoes search modal Importacao
    public async createPeriodoModal() {
      this.modalImportacao = await ModalService.createFromJson('/modals/gridPeriodoImportCard', true);
    }
    
    public openModalImportacao() {
      this.modalImportacao.show();
    }
    
    public destroyPeriodoModal() {
      this.modalImportacao.destroy();
    }
    
    public closeModal() {
      this.modalImportacao.hide();
    }
    // ---------
    
    public async createModal() {
      this.modalImportCard = await ModalService.createFromJson('modals/pme00602_importar_cardapio_dia', true);
    }
    
    public fullscreen({ component }: IButtonEvent) {
      const modal: IDictionary = Metadata.getInstance('modalHeader');
      modal.parent.fullscreen = !modal.parent.fullscreen;
      component.iconName = (modal.parent.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
    }
    
    public cancelModal() {
      if (this.modalImportCard) {
        this.modalImportCard.destroy();
        this.destroyPeriodoModal();
        this.colsGridPara = 12;
        this.showGridPrato = false;
        this.NRCARDPMEORIGEM = '';
        this.datasourceDePara = [];
        const cardapio: Cardapio = Metadata.getInstance('cardapio');
        cardapio.setValueAction(false);
      }
    }
    
    public showModal({ event }: IEventParam<Button>) {
      (event as IDictionary).stopPropagation();
      this.validAndOpenModal();
    }
    
    public async validAndOpenModal() {
      if (this.elaboracaoCardapio.getFilterData.NRCARDPME) {
        this.cancelModal();
        const cardapio: Cardapio = Metadata.getInstance('cardapio');
        cardapio.setValueAction(true);
        await this.createModal();
        await this.createPeriodoModal();
        this.modalImportCard.show();
      } else {
        this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
      }
    }
    
    public setDefaultDates() {
      const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
      const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
      const dtDe = Metadata.getInstance<ZdDate>('DTDE');
      const dtAte = Metadata.getInstance<ZdDate>('DTATE');
      
      periodoDe.value = this.elaboracaoCardapio.getFilterData.initialDate;
      periodoAte.value = this.elaboracaoCardapio.getFilterData.finalDate;
      dtDe.value = this.elaboracaoCardapio.getFilterData.initialDate;
      dtAte.value = this.elaboracaoCardapio.getFilterData.finalDate;
      this.NRCARDPMEORIGEM = this.elaboracaoCardapio.getFilterData.NRCARDPME;
    }
    
    public clearFields({ component }: IEventParam<Select>) {
      const cardapio = Metadata.getInstance<Select>('TIPOCARD');
      const servico = Metadata.getInstance<Select>('SERVICO');
      
      if (component.name === 'CENTRAL') cardapio.setValue('');
      
      servico.setValue('');
      this.setDefaultDates();
    }
    
    public setFilterTpCardapio({ component }: IEventParam<Select>) {
      const central = Metadata.getInstance<Select>('CENTRAL');
      const filter: IDictionary = [];
      this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', central.value);
      this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
      
      component.datasource.filter = filter;
      component.datasource.get();
    }
    
    public hasShowArredondar() {
      const IDPERCCOME = (localStorage.getItem('IDPERCCOME') === null) ? 'C' : localStorage.getItem('IDPERCCOME');
      this.showArredondar = (IDPERCCOME === 'C');
    }
    
    public setFilterServ({ component }: IEventParam<Select>) {
      const central = Metadata.getInstance<Select>('CENTRAL');
      const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
      const filter = [
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: tipoCard.value ? tipoCard.value : '',
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: central.value,
        },
      ];
      
      component.datasource.filter = filter;
      component.datasource.get();
    }
    
    public openModalSearch() {
      const tipoCard = Metadata.getInstance<Select>('TIPOCARD').value;
      if (tipoCard) {
        this.openModalImportacao();
        this.utilities.setCdTipoCard(tipoCard);
        this.utilities.setFilterModalPeriodo();
      } else {
        this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
      }
    }
    
    public setPeriodoParams({ row }: IDictionary) {
      const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
      const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
      const dtDe = Metadata.getInstance<ZdDate>('DTDE');
      const dtAte = Metadata.getInstance<ZdDate>('DTATE');
      periodoDe.value = row.DTINICARD;
      periodoAte.value = row.DTFINCARD;
      dtDe.value = row.DTINICARD;
      dtAte.value = row.DTFINCARD;
      this.NRCARDPMEORIGEM = row.NRCARDPME;
      this.modalImportacao.hide();
    }
    
    public setRadioDefault({ component }: IEventParam<Radio>) {
      component.value = (component.datasource as IDictionary).data[0].value;
    }
    
    // Quando mudar as funções de data para change habilitar essa função novamente;
    // public setDefaultInclude({ component }: IEventParam<ZdDate>) {
    //   const cardapio: Cardapio = Metadata.getInstance('cardapio');
    //   component.value = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
    // }
    
    public filterGridPara() {
      const form1 = Metadata.getInstance<Form>('form1').validate();
      const form2 = Metadata.getInstance<Form>('form2').validate();
      const form3 = Metadata.getInstance<Form>('form3').validate();
      
      if (form1 && form2 && form3) {
        const filter: IDictionary = [];
        const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined) ? null : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
        this.utilities.onMountedFilterFind(filter, 'P_NRCARDPME', '=', this.NRCARDPMEORIGEM);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', Metadata.getInstance<Select>('CENTRAL').value);
        this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', Metadata.getInstance<Select>('SERVICO').value);
        this.utilities.onMountedFilterFind(filter, 'P_CDFILAGREG', '=', filial);
        this.utilities.onMountedFilterFind(filter, 'P_DTINICIAL', '=', Metadata.getInstance<ZdDate>('PERIODODE').value);
        this.utilities.onMountedFilterFind(filter, 'P_DTFINAL', '=', Metadata.getInstance<ZdDate>('PERIODOATE').value);
        
        Http.post('/find/datasDePara', { requestType: 'FilterData', filter, limit: 1000000 }).then((dePara) => {
          this.mountDataSourceDePara(dePara.data.data);
        });
      }
    }
    
    private mountDataSourceDePara(dePara: IDictionary) {
      this.datasourceDePara = [];
      const dtDe = Metadata.getInstance<ZdDate>('DTDE');
      const dtAte = Metadata.getInstance<ZdDate>('DTATE');
      const incluirCardDe = Metadata.getInstance<ZdDate>('DTINCLUIR');
      
      const idxInicial = dePara.findIndex((data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === dtDe.value);
      const idxFinal = dePara.findIndex((data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === dtAte.value);
      
      const newDePara = dePara.slice(idxInicial, (idxFinal + 1));
      
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const idxInicialCard = cardapio.dates.findIndex((data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === incluirCardDe.value);
      const newDateCard = cardapio.dates.slice(idxInicialCard, cardapio.dates.length);
      
      newDateCard.forEach((object: IDictionary, idx: number) => {
        if (newDePara.length > idx) {
          const template: IDictionary = {};
          template.DE = moment(newDePara[idx].DTCARDIAPME, 'DD/MM/YYYY').format(this.getFormatLocaleFullWeek());
          template.PARA = moment(object.DTCARDIAPME, 'DD/MM/YYYY').format(this.getFormatLocaleFullWeek());
          this.datasourceDePara.push(template);
        }
      });
      
      if (this.datasourceDePara.length > 0) {
        this.requestCorDePara(this.datasourceDePara).then((corDePara) => {
          if (corDePara.length > 0) this.attributeColor(corDePara, this.datasourceDePara);
          const gridDePara = Metadata.getInstance<Grid>('gridDePara');
          gridDePara.datasource.updateData(this.datasourceDePara).then((data: IDictionary) => {
            gridDePara.datasource.currentRow = data[0];
            if (!this.showGridPrato) this.changeVisibilityGridPrato();
            this.filterGridPratoCardDia({ row: data[0] });
          });
        });
      }
    }
    
    // PEGA A COR DAS DATAS DE-PARA
    private requestCorDePara(datasource: IDictionary) {
      const filter: IDictionary = [];
      const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined) ? Metadata.getInstance<Select>('CENTRAL').value : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
      this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', filial);
      this.utilities.onMountedFilterFind(filter, 'P_DTINIORIG', '=', moment(datasource[0].DE.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'));
      this.utilities.onMountedFilterFind(filter, 'P_DTFIMORIG', '=', moment(datasource[datasource.length - 1].DE.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'));
      this.utilities.onMountedFilterFind(filter, 'P_DTINIDEST', '=', moment(datasource[0].PARA.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'));
      this.utilities.onMountedFilterFind(filter, 'P_DTFIMDEST', '=', moment(datasource[datasource.length - 1].PARA.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'));
      
      return Http.post('/find/corDePara', { requestType: 'FilterData', filter, limit: 1000000 }).then((corDePara) => {
        const data = corDePara.data.data;
        return data;
      });
    }
    
    public filterGridPratoCardDia({ row }: IDictionary) {
      const gridPrato = Metadata.getInstance<Grid>('gridPrato');
      const filter: IDictionary = [];
      const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined) ? Metadata.getInstance<Select>('CENTRAL').value : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
      this.utilities.onMountedFilterFind(filter, 'P_NRCARDPME', '=', this.NRCARDPMEORIGEM);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', Metadata.getInstance<Select>('CENTRAL').value);
      this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', Metadata.getInstance<Select>('SERVICO').value);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILAGREG', '=', filial);
      this.utilities.onMountedFilterFind(filter, 'P_DTCARDIAPME', '=', moment(row.DE.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'));
      
      gridPrato.datasource.filter = filter;
      gridPrato.datasource.get();
    }
    
    // ATRIBUI A COLORAÇÃO DE ACORDO COM OQUE VEM DO BANCO.
    private attributeColor(corDePara: IDictionary, datasource: IDictionary) {
      corDePara.forEach((cor: IDictionary) => {
        const hasData = datasource.find((data: IDictionary) => cor.DTFERIFILI.substring(0, 10) === data.DE.substring(0, 10));
        if (hasData) {
          hasData.IDCARDIAUTIL = cor.IDCARDIAUTIL;
          hasData.NRCORNUTIL = cor.NRCORNUTIL;
        }
      });
    }
    
    public coloredText({ row }: IDictionary) {
      if (row.IDCARDIAUTIL === undefined || row.IDCARDIAUTIL === 'N') return '';
      
      let color = row.NRCORNUTIL;
      color = this.utilities.TColorToHex(color);
      color = color.substring(1, color.length);
      if (color.length < 6) color = this.utilities.increamentZeroInHex(color);
      return `#${color}`;
    }
    
    public getFormatLocaleFullWeek(): string {
      return (localStorage.getItem('LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY dddd' : 'MM/DD/YYYY dddd';
    }
    
    private changeVisibilityGridPrato() {
      this.showGridPrato = !this.showGridPrato;
      this.colsGridPara = (this.showGridPrato) ? 6 : 12;
    }
    
    // IMPORTAÇÃO DO CARDAPIO
    public importarCardapio({ component }: IButtonEvent) {
      const form1 = Metadata.getInstance<Form>('form1').validate();
      const form2 = Metadata.getInstance<Form>('form2').validate();
      const form3 = Metadata.getInstance<Form>('form3').validate();
      
      if (form1 && form2 && form3) {
        if (this.datasourceDePara.length > 0) {
          const row: IDictionary = {};
          row.formImport = this.mountFormImport();
          row.filterData = this.elaboracaoCardapio.getFilterData;
          row.dePara = this.mountDeParaImport();
          this.utilities.activeLoadingButton(component);
          Http.post('/importarCardapio', { requestType: 'Row', row }).then((response) => {
            if (response.data.dataset.successImport[0]) {
              this.cancelModal();
              this.utilities.showAlert('successImport', 'pme::sucessoNaImportacao', 'success');
              this.elaboracaoCardapio.reloadCard();
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          }).finally(() => {
            this.utilities.disableLoadingButton(component);
          });
        } else {
          this.utilities.showAlert('cardapioOrigem', 'pme::naoPossuiCardapioOrigem', 'error');
        }
      }
    }
    
    private mountFormImport() {
      const form3 = Metadata.getInstance<Form>('form3');
      return {
        CENTRAL: Metadata.getInstance<Form>('form1').value.CENTRAL,
        TIPOCARD: Metadata.getInstance<Form>('form2').value.TIPOCARD,
        SERVICO: form3.value.SERVICO,
        NRCARDPME: this.NRCARDPMEORIGEM,
        ARREDONDAR: form3.value.arredondarComen,
        RADIOSELECT: form3.value.radioSelect,
      };
    }
    
    // VERIFICA SE AS DATAS QUE IRAM SER IMPORTADAS SÃO FERIADO OU BLOQUEADAS. PARA PODER ENVIAR APENAS AS DATAS CORRETAS.
    private mountDeParaImport() {
      const dePara: IDictionary[] = [];
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      
      this.datasourceDePara.forEach((data: IDictionary) => {
        const template: IDictionary = {};
        const dataPara = moment(data.PARA.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY');
        const dataCard = cardapio.dates.find((dataCardapio: IDictionary) => dataCardapio.DTCARDIAPME.substring(0, 10) === dataPara);
        if (dataCard.DTFERIFILI == null && cardapio.dateLessDateOfProcess(dataCard.DTCARDIAPME)) {
          template.DE = moment(data.DE.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY');
          template.PARA = dataPara;
          dePara.push(template);
        }
      });
      
      return dePara;
    }
    
    public validData() {
      const format = 'DD/MM/YYYY';
      const periodoDe = moment(Metadata.getInstance<ZdDate>('PERIODODE').value, format);
      const periodoAte = moment(Metadata.getInstance<ZdDate>('PERIODOATE').value, format);
      const dtDe = moment(Metadata.getInstance<ZdDate>('DTDE').value, format);
      const dtAte = moment(Metadata.getInstance<ZdDate>('DTATE').value, format);
      
      if ((dtDe < periodoDe || dtDe > periodoAte) || (dtAte < dtDe || dtAte > periodoAte)) {
        this.utilities.showAlert('foraDoPeriodo', 'pme::dataForaDoPeriodoSelecionado', 'error');
        this.resetAllGrid();
      } else {
        this.filterGridPara();
      }
    }
    
    public validaDataIncluir({ component }: IEventParam<ZdDate>): boolean {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const hasDtIncluir = cardapio.dates.find((data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === component.value);
      
      if (hasDtIncluir === undefined) {
        this.utilities.showAlert('foraDoPeriodo', 'pme::dataDeInclusaoForaDoPeriodo', 'error');
        this.resetAllGrid();
      } else {
        this.filterGridPara();
      }
      
      return true;
    }
    
    private resetAllGrid() {
      this.datasourceDePara = [];
      const gridDePara = Metadata.getInstance<Grid>('gridDePara');
      gridDePara.datasource.updateData(this.datasourceDePara);
      this.changeVisibilityGridPrato();
    }
}
